<?php
/*
Plugin Name: Recently Popular
Description: Displays the most popular posts based on history from now to X amount of time in the past.
Author: Eric Biven
Author URI: http://eric.biven.us/
Plugin URI: http://eric.biven.us/2008/12/03/recently-popular-wordpress-plugin/
Version: 0.4.7
*/

// Copyright (c) 2008 Eric Biven
// Released under the FreeBSD license:
// http://www.freebsd.org/copyright/freebsd-license.html

// This type of pathing is required for my development environment.
require_once(ABSPATH.'wp-content/plugins/recently-popular/include.php');

function recently_popular_activate() {
    global $wpdb;
    $table = $wpdb->prefix.'recently_popular';

    $sql = "CREATE TABLE $table (
        ts TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        user_type TINYINT UNSIGNED NOT NULL DEFAULT 1,
        post_id BIGINT(20) UNSIGNED NOT NULL,
        INDEX index_all(ts, user_type, post_id)
        );";

    // This type of pathing is required for my development environment.
    require_once(ABSPATH.'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}
register_activation_hook(ABSPATH.'wp-content/plugins/recently-popular/recently_popular.php', 'recently_popular_activate');

function recently_popular_record_hit() {
    // Only record the view if it is a post or a page.  This keeps us from recording listings pages.
    if (is_single() || is_page()) {
        global $wpdb, $user_ID, $post;
        $user_type = (empty($_COOKIE[USER_COOKIE]) && intval($user_ID) == 0) ? RP_USER_TYPE_ANONYMOUS : RP_USER_TYPE_REGISTERED;
        $table = $wpdb->prefix.'recently_popular';
        $wpdb->query("INSERT INTO $table(`user_type`, `post_id`) VALUES ($user_type, $post->ID);");
    }
}
add_action('wp_head', 'recently_popular_record_hit');

function get_recently_popular(
                $interval_length = 1,
                $interval_type = 'MONTH',
                $limit = 10,
                $user_type = RP_USER_TYPE_ANONYMOUS,
                $post_type = RP_POST_TYPE_ALL,
                $output_format = '<a href="%post_url%">%post_title%</a>',
                $categories = '',
                $display = true) {

    $output = '';
    global $wpdb;
    $table = $wpdb->prefix.'recently_popular';

    // Establish the needed where clauses
    $wc_user_type = '';
    $wc_post_type = '';
    $wc_categories = '';
    $join_categories = '';

    if ($user_type != RP_USER_TYPE_ALL) {
        $wc_user_type = " AND rp.`user_type` = $user_type ";
    }

    switch ($post_type) {
      case RP_POST_TYPE_ALL:
        break;
      case RP_USER_TYPE_PAGES:
        $wc_post_type = " AND p.`post_type` = 'page' ";
        break;
      case RP_USER_TYPE_POSTS:
        $wc_post_type = " AND p.`post_type` = 'post' ";
        break;
    }

    if (strlen($categories) > 0) {
        // Define the where-clause (wc) and join this way so that we don't
        // join the term tables unless we need to.
        $categories = stripslashes($categories);
        $join_categories = " LEFT JOIN $wpdb->term_relationships AS tr ON p.`ID` = tr.`object_id`
                             LEFT JOIN $wpdb->term_taxonomy AS tt ON tr.`term_taxonomy_id` = tt.`term_taxonomy_id`
                             LEFT JOIN $wpdb->terms AS t ON tt.`term_id` = t.`term_id`";

        // Since pages can't have categories we have to use an appropriate
        // where clause if the user wants them displayed.
        if ($post_type == RP_USER_TYPE_POSTS) {
            $wc_categories = " AND tt.`taxonomy` = 'category'
                               AND t.`name` IN ($categories)";
        }
        else {
            $wc_categories = " AND ((tt.`taxonomy` = 'category'
                                     AND t.`name` IN ($categories))
                                    OR p.`post_type` = 'page')";
        }
    }

    $sql = "SELECT COUNT(rp.`post_id`) AS hits,
            MAX(rp.`post_id`) AS `post_id`,
            MAX(p.`post_title`) AS `post_title`,
            MAX(u.`display_name`) AS `display_name`,
            MAX(u.`user_url`) AS `user_url`
            FROM $table AS rp
            LEFT JOIN $wpdb->posts AS p ON rp.`post_id` = p.`ID`
            LEFT JOIN $wpdb->users AS u ON p.`post_author` = u.`ID`
            $join_categories
            WHERE rp.`ts` > (CURRENT_TIMESTAMP() - INTERVAL $interval_length $interval_type)
              $wc_user_type
              $wc_post_type
              $wc_categories
            GROUP BY rp.`post_id`
            ORDER BY hits DESC, rp.`ts` DESC
            LIMIT $limit;
    ";

    $most_viewed = $wpdb->get_results($sql);

    if ($most_viewed) {
        foreach ($most_viewed as $post) {
            $loutput = str_replace('%post_title%', $post->post_title, $output_format);
            $loutput = str_replace('%post_url%', get_permalink($post->post_id), $loutput);
            $loutput = str_replace('%hits%', $post->hits, $loutput);
            $loutput = str_replace('%display_name%', $post->display_name, $loutput);
            $loutput = str_replace('%user_url%', $post->user_url, $loutput);

            $output .= "    <li>$loutput</li>\n";
        }
    }

    if ($display) { echo $output; }
    else { return $output; }
}
?>
